@@ -1,3 +1,59 @@
+//! Generates CPython CFFI bindings for [Interoptopus](https://github.com/ralfbiedert/interoptopus).
+//!
+//! ## Usage
+//!
+//! In your library or a support project add this:
+//!
+//! ```
+//! # mod my_crate { use interoptopus::{Library}; pub fn ffi_inventory() -> Library { todo!() } }
+//! use my_crate::ffi_inventory;
+//!
+//! #[test]
+//! fn generate_python_bindings() {
+//!     use interoptopus::Interop;
+//!     use interoptopus_backend_cpython_cffi::{Generator, InteropCPythonCFFI, Config};
+//!
+//!     // Converts an `ffi_inventory()` into Python interop definitions.
+//!     Generator::new(Config::default(), ffi_inventory()).write_to("module.py")
+//! }
+//! ```
+//!
+//! And we might produce something like this:
+//!
+//! ```python
+//! from cffi import FFI
+//!
+//! api_definition = """
+//! typedef struct Vec3f32
+//!     {
+//!     float x;
+//!     float y;
+//!     float z;
+//!     } Vec2f32;
+//!
+//! Vec3f32 my_game_function(Vec3f32* input);
+//! """
+//!
+//!
+//! ffi = FFI()
+//! ffi.cdef(api_definition)
+//! _api = None
+//!
+//!
+//! def init_api(dll):
+//!     """Initializes this library, call with path to DLL."""
+//!     global _api
+//!     _api = ffi.dlopen(dll)
+//!
+//!
+//! class raw:
+//!     """Raw access to all exported functions."""
+//!
+//!     def my_game_function(input):
+//!     global _api
+//!     return _api.my_game_function(input)
+//! ```
+
 use interoptopus::lang::c::{CType, ConstantValue, EnumType, FnPointerType, Function, PrimitiveValue};
 use interoptopus::patterns::class::Class;
 use interoptopus::patterns::{LibraryPattern, TypePattern};
@@ -7,12 +63,17 @@ use interoptopus::Interop;
 use interoptopus::{Error, Library};
 use interoptopus_backend_c::InteropC;
 
+/// Configures Python code generation.
 #[derive(Clone, Debug)]
 pub struct Config {
-    init_api_function_name: String,
-    ffi_attribute: String,
-    raw_fn_namespace: String,
-    callback_namespace: String,
+    /// How to name the function responsible for loading the DLL, e.g., `init_api`.
+    pub init_api_function_name: String,
+    /// Attribute by which the `cffi` object is exposed, e.g., `ffi`.
+    pub ffi_attribute: String,
+    /// Namespace to put functions into, e.g., `raw`.
+    pub raw_fn_namespace: String,
+    /// Namespace for callback helpers, e.g., `callbacks`.
+    pub callback_namespace: String,
 }
 
 impl Default for Config {
@@ -26,6 +87,7 @@ impl Default for Config {
     }
 }
 
+/// Helper type implementing [`InteropCPythonCFFI`] and [`Interop`].
 pub struct Generator {
     c_generator: interoptopus_backend_c::Generator,
     config: Config,
@@ -48,6 +110,7 @@ impl Generator {
     }
 }
 
+/// Contains all Python generators, create sub-trait to customize.
 pub trait InteropCPythonCFFI: Interop {
     /// Returns the user config.
     fn config(&self) -> &Config;
@@ -102,7 +165,14 @@ pub trait InteropCPythonCFFI: Interop {
 
     fn write_constants(&self, w: &mut IndentWriter) -> Result<(), Error> {
         for constant in self.library().constants() {
-            let docs = constant.documentation().lines().iter().map(|x| format!("# {}", x)).collect::<Vec<_>>().join("\n");
+            let docs = constant
+                .meta()
+                .documentation()
+                .lines()
+                .iter()
+                .map(|x| format!("# {}", x))
+                .collect::<Vec<_>>()
+                .join("\n");
             w.indented(|w| writeln!(w, r#"{}"#, docs))?;
             w.indented(|w| writeln!(w, r#"{} = {}"#, constant.name(), self.constant_value_to_value(constant.value())))?;
         }
@@ -123,7 +193,7 @@ pub trait InteropCPythonCFFI: Interop {
     }
 
     fn write_enum(&self, w: &mut IndentWriter, e: &EnumType) -> Result<(), Error> {
-        let docs = e.documentation().lines().join("\n");
+        let docs = e.meta().documentation().lines().join("\n");
 
         w.indented(|w| writeln!(w, r#"class {}:"#, e.rust_name()))?;
         w.indent();
@@ -177,7 +247,7 @@ pub trait InteropCPythonCFFI: Interop {
 
         for function in self.library().functions() {
             let args = function.signature().params().iter().map(|x| x.name().to_string()).collect::<Vec<_>>().join(", ");
-            let docs = function.documentation().lines().join("\n");
+            let docs = function.meta().documentation().lines().join("\n");
 
             w.indented(|w| writeln!(w, r#"def {}({}):"#, function.name(), &args))?;
             w.indent();
@@ -254,7 +324,7 @@ pub trait InteropCPythonCFFI: Interop {
 
         // Ctor
         let args = self.pattern_class_args_without_first_to_string(class.constructor());
-        let docs = class.constructor().documentation().lines().join("\n");
+        let docs = class.constructor().meta().documentation().lines().join("\n");
         w.indented(|w| writeln!(w, r#"def __init__(self, {}):"#, args))?;
         w.indent();
         w.indented(|w| writeln!(w, r#""""{}""""#, docs))?;
@@ -265,7 +335,7 @@ pub trait InteropCPythonCFFI: Interop {
         w.newline()?;
 
         // Dtor
-        let docs = class.destructor().documentation().lines().join("\n");
+        let docs = class.destructor().meta().documentation().lines().join("\n");
         w.indented(|w| writeln!(w, r#"def __del__(self):"#))?;
         w.indent();
         w.indented(|w| writeln!(w, r#""""{}""""#, docs))?;
@@ -276,7 +346,7 @@ pub trait InteropCPythonCFFI: Interop {
 
         for function in class.methods() {
             let args = self.pattern_class_args_without_first_to_string(function);
-            let docs = function.documentation().lines().join("\n");
+            let docs = function.meta().documentation().lines().join("\n");
 
             w.indented(|w| writeln!(w, r#"def {}(self, {}):"#, function.name().replace(&common_prefix, ""), &args))?;
             w.indent();
